addpath('tools');
clear all;

mu = 0;                 % Mean of the distributions
sigma1 = 1;             % First standard deviation value for the distributions
sigma2 = 3 * sqrt(2);   % Second standard deviation value for the distributions
n_samples = 100;        % Number of samples to sample from each distribution
n = 2000;               % Number of times to independently compute the MMD
kernel_sigma = 0.5;     % The kernel width to use

%% Compute multiple MMD values using several samples for 6 different cases
mmd_normal_null = zeros(n, 1);
mmd_normal_alternative = zeros(n, 1);
mmd_laplace_null = zeros(n, 1);
mmd_laplace_alternative = zeros(n, 1);
mmd_normal_laplace_same_sigma = zeros(n, 1);
mmd_normal_laplace_different_sigma = zeros(n, 1);
for i = 1:n
    % Sample from the distributions
    normal1_sample1 = normrnd(mu, sigma1, n_samples, 1);
    normal1_sample2 = normrnd(mu, sigma1, n_samples, 1);
    normal2_sample = normrnd(mu, sigma2, n_samples, 1);
    laplace1_sample1 = laprnd(mu, sigma1, n_samples, 1);
    laplace1_sample2 = laprnd(mu, sigma1, n_samples, 1);
    laplace2_sample = laprnd(mu, sigma2, n_samples, 1);
    % Compute the MMD values
    mmd_normal_null(i) = mmd(normal1_sample1, normal1_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
    mmd_normal_alternative(i) = mmd(normal1_sample1, normal2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
    mmd_laplace_null(i) = mmd(laplace1_sample1, laplace1_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
    mmd_laplace_alternative(i) = mmd(laplace1_sample1, laplace2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
    mmd_normal_laplace_same_sigma(i) = mmd(normal1_sample1, laplace1_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
    mmd_normal_laplace_different_sigma(i) = mmd(normal1_sample1, laplace2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
end

%% Plot the resulting histograms (i.e., empirical distributions) of the MMD values
figure('Units', 'normalized', 'Position', [.1 .1 .5 .7]);
subplot(3, 2, 1);
hist(mmd_normal_null, 20);
title('\textbf{Null:} $$\mathcal{N}(0,1)$$ vs. $$\mathcal{N}(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
subplot(3, 2, 2);
hist(mmd_normal_alternative, 20);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. $$\mathcal{N}(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
subplot(3, 2, 3);
hist(mmd_laplace_null, 20);
title('\textbf{Null:} Laplace$$(0,1)$$ vs. Laplace$$(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
subplot(3, 2, 4);
hist(mmd_laplace_alternative, 20);
title('\textbf{Alternative:} Laplace$$(0,1)$$ vs. Laplace$$(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
subplot(3, 2, 5);
hist(mmd_normal_laplace_same_sigma, 20);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. Laplace$$(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
subplot(3, 2, 6);
hist(mmd_normal_laplace_different_sigma, 20);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. Laplace$$(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Probability Density');
% Save the plot
set(gcf, 'PaperPositionMode', 'auto');
print -depsc2 plots/plot_3_empirical_MMD_distributions.eps;
fixPSlinestyle('plots/plot_3_empirical_MMD_distributions.eps', 'plots/plot_3_empirical_MMD_distributions.eps');

%% Compute the empirical MMD CDFs and the Pearson approximation CDFs
alpha = 0.05;                   % The alpha value used for hypothesis testing
numberOfPearsonSamples = 1000;  % The number of samples to draw from the Pearson system when estimating the MMD CDF

[mmd_normal_null_empirical_cdf, mmd_normal_null_empirical_cdf_x] = ecdf(mmd_normal_null);
[~, mmd_normal_null_pearson_cdf, mmd_normal_null_pearson_cdf_x, ~] = ...
    mmd_pearson_test(normal1_sample1, normal1_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
[mmd_normal_alternative_empirical_cdf, mmd_normal_alternative_empirical_cdf_x] = ecdf(mmd_normal_alternative);
[~, mmd_normal_alternative_pearson_cdf, mmd_normal_alternative_pearson_cdf_x, ~] = ...
    mmd_pearson_test(normal1_sample1, normal2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
[mmd_laplace_null_empirical_cdf, mmd_laplace_null_empirical_cdf_x] = ecdf(mmd_laplace_null);
[~, mmd_laplace_null_pearson_cdf, mmd_laplace_null_pearson_cdf_x, ~] = ...
    mmd_pearson_test(laplace1_sample1, laplace1_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
[mmd_laplace_alternative_empirical_cdf, mmd_laplace_alternative_empirical_cdf_x] = ecdf(mmd_laplace_alternative);
[~, mmd_laplace_alternative_pearson_cdf, mmd_laplace_alternative_pearson_cdf_x, ~] = ...
    mmd_pearson_test(laplace1_sample1, laplace2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
[mmd_normal_laplace_same_sigma_empirical_cdf, mmd_normal_laplace_same_sigma_empirical_cdf_x] = ecdf(mmd_normal_laplace_same_sigma);
[~, mmd_normal_laplace_same_sigma_pearson_cdf, mmd_normal_laplace_same_sigma_pearson_cdf_x, ~] = ...
    mmd_pearson_test(normal1_sample1, laplace1_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
[mmd_normal_laplace_different_sigma_empirical_cdf, mmd_normal_laplace_different_sigma_empirical_cdf_x] = ecdf(mmd_normal_laplace_different_sigma);
[~, mmd_normal_laplace_different_sigma_pearson_cdf, mmd_normal_laplace_different_sigma_pearson_cdf_x, ~] = ...
    mmd_pearson_test(normal1_sample1, laplace2_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);

%% Plot the empirical MMD CDFs and the Pearson approximation CDFs
figure('Units', 'normalized', 'Position', [.1 .1 .5 .7]);
subplot(3, 2, 1);
hold on;
plot(mmd_normal_null_empirical_cdf_x, mmd_normal_null_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_normal_null_pearson_cdf_x, mmd_normal_null_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Null:} $$\mathcal{N}(0,1)$$ vs. $$\mathcal{N}(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
subplot(3, 2, 2);
hold on;
plot(mmd_normal_alternative_empirical_cdf_x, mmd_normal_alternative_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_normal_alternative_pearson_cdf_x, mmd_normal_alternative_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. $$\mathcal{N}(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
subplot(3, 2, 3);
hold on;
plot(mmd_laplace_null_empirical_cdf_x, mmd_laplace_null_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_laplace_null_pearson_cdf_x, mmd_laplace_null_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Null:} Laplace$$(0,1)$$ vs. Laplace$$(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
subplot(3, 2, 4);
hold on;
plot(mmd_laplace_alternative_empirical_cdf_x, mmd_laplace_alternative_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_laplace_alternative_pearson_cdf_x, mmd_laplace_alternative_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Alternative:} Laplace$$(0,1)$$ vs. Laplace$$(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
subplot(3, 2, 5);
hold on;
plot(mmd_normal_laplace_same_sigma_empirical_cdf_x, mmd_normal_laplace_same_sigma_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_normal_laplace_same_sigma_pearson_cdf_x, mmd_normal_laplace_same_sigma_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. Laplace$$(0,1)$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
subplot(3, 2, 6);
hold on;
plot(mmd_normal_laplace_different_sigma_empirical_cdf_x, mmd_normal_laplace_different_sigma_empirical_cdf, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(mmd_normal_laplace_different_sigma_pearson_cdf_x, mmd_normal_laplace_different_sigma_pearson_cdf, 'r', 'LineWidth', 1.5);
title('\textbf{Alternative:} $$\mathcal{N}(0,1)$$ vs. Laplace$$(0,3\sqrt{2})$$', 'interpreter', 'latex');
xlabel('$$\textnormal{MMD }_u^2$$', 'interpreter', 'latex');
ylabel('Cumulative Density Function (CDF)');
legend('Empirical CDF', 'Pearson CDF');
hold off;
% Save the plot
set(gcf, 'PaperPositionMode', 'auto');
print -depsc2 plots/plot_4_MMD_CDF_comparisons.eps;
fixPSlinestyle('plots/plot_4_MMD_CDF_comparisons.eps', 'plots/plot_4_MMD_CDF_comparisons.eps');