function [mmd, test_threshold] = ...
    mmd_bootstrap_test(x1, x2, kernel, alpha, biased, iterations)
% mmd_bootstrap_test computes the MMD value for samples x1 and x2, using 
% kernel as the kernel function and it then uses a bootstrapping procedure 
% to compute a threshold for performing hypothesis testing using parameter 
% alpha as the significance level. The null hypothesis being tested is 
% assuming that x1 and x2 were generated from the same distribution. The
% iterations argument specifies the number of bootstrapping iterations to
% use, and the biased argument is a boolean variable indicating whether to
% compute and use a biased or an unbiased MMD estimate.

n = size(x1, 1);

% Compute an MMD estimate
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
if (biased)
    % Compute a biased MMD estimate
    mmd = sum(sum(mmd_sum_terms)) / (n^2);
else
    % Compute the unbiased MMD estimate
    mmd_sum_terms = mmd_sum_terms - diag(diag(mmd_sum_terms));
    mmd = sum(sum(mmd_sum_terms)) / (n*(n-1));
end

% Perform the bootstrapping step
kernels_matrix = [K KL; KL' L];
shuffled_MMDs = zeros(iterations, 1);
for i = 1:iterations
    [~, indexes] = sort(rand(2*n, 1));
    shuffled_kernels_matrix = kernels_matrix(indexes,indexes);
    K = shuffled_kernels_matrix(1:n,1:n);
    L = shuffled_kernels_matrix(n+1:2*n,n+1:2*n);
    KL = shuffled_kernels_matrix(1:n,n+1:2*n);
    mmd_sum_terms = (K + L - KL - KL');
    if (biased)
        % Compute a biased MMD estimate
        shuffled_MMDs(i) = sum(sum(mmd_sum_terms)) / (n^2);
    else
        % Compute the unbiased MMD estimate
        mmd_sum_terms = mmd_sum_terms - diag(diag(mmd_sum_terms));
        shuffled_MMDs(i) = sum(sum(mmd_sum_terms)) / (n*(n-1));
    end
end
% Compute the test threshold
shuffled_MMDs = sort(shuffled_MMDs);
test_threshold = shuffled_MMDs(round((1-alpha) * iterations));

end