function [mmd, test_threshold] = mmd_gamma_test(x1, x2, kernel, alpha)
% mmd_gamma_test computes the MMD value for samples x1 and x2, using kernel
% as the kernel function and it then uses the Gamma approximation to 
% compute a threshold for performing hypothesis testing using parameter 
% alpha as the significance level. The null hypothesis being tested is 
% assuming that x1 and x2 were generated from the same distribution. Note
% that this test computes and uses the biased MMD estimate and its
% computation time is O(n^2) where n is the number of samples used. The
% test that uses the Pearson curves approximation computes and uses the
% unbiased MMD estimate, but it has a computation time of O(n^3).

n = size(x1, 1);

% Compute a biased MMD estimate
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
mmd = sum(sum(mmd_sum_terms)) / (n^2);

% Compute the mean of the MMD under the null hypothesis
mmd_mean  = 2 / n * (1 - sum(diag(KL)/n));
% Compute the variance of the MMD under the null hypothesis
K = K - diag(diag(K));
L = L - diag(diag(L));
KL = KL - diag(diag(KL));
mmd_variance = 2 / (n^2*(n-1)^2) * sum(sum((K + L - KL - KL') .^ 2));
% Compute the Gamma distribution parameters
gamma_alpha = mmd_mean^2 / mmd_variance;
gamma_beta = mmd_variance * n / mmd_mean;

% Compute the test threshold
test_threshold = icdf('gam', 1 - alpha, gamma_alpha, gamma_beta) / n;

end