function [mmd, mmd_empirical_cdf, mmd_empirical_cdf_x, test_threshold] = ...
    mmd_pearson_test(x1, x2, kernel, alpha, numberOfPearsonSamples)
% mmd_pearson_test computes an unbiased MMD estimate for samples x1 and x2,
% using kernel as the kernel function and it then uses the Pearson curves 
% approximation to approximate the CDF of the MMD and compute a threshold 
% for performing hypothesis testing using parameter alpha as the 
% significance level. The null hypothesis being tested is assuming that x1 
% and x2 were generated from the same distribution. When computing the 
% Pearson approximation, numberOfPearsonSamples samples are drawn from the 
% Pearson system. This approximation for testing has a computation time of 
% O(n^3) where n is the number of samples used. The test that uses the 
% Gamma approximation has a computation time of O(n^2), but it computes and
% uses a biased MMD estimate.

n = size(x1, 1);

% Compute the unbiased MMD estimate
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
mmd_sum_terms = mmd_sum_terms - diag(diag(mmd_sum_terms));
mmd = sum(sum(mmd_sum_terms)) / (n*(n-1));

% Compute the 2nd moment of the MMD
U_m2 = sum(sum((mmd_sum_terms .^ 2) / (n*(n-1)))) * 2 / (n*(n-1));
% Compute the 3rd moment of the MMD
U_m3 = 0;
for i1 = 1:n
    for i2 = 1:n
        if i1 ~= i2
            intermediate_mean = 0;
            for i3 = 1:n
                if i3 ~= i1 && i3 ~= i2
                    intermediate_mean = intermediate_mean + ...
                        mmd_sum_terms(i1, i3) * mmd_sum_terms(i2, i3);
                end
            end
            intermediate_mean = intermediate_mean / (n - 2);
            U_m3 = U_m3 + intermediate_mean * mmd_sum_terms(i1, i2);
        end
    end
end
U_m3 = U_m3 / (n*(n-1));
U_m3 = 8 * (n-2) / (n*(n-1))^2 * U_m3;
% Approximate the 4th moment of the MMD using the bound: 
% kurtosis >= skewness^2 + 1
kurtosis = 2 * ((U_m3)^2 / U_m2^3 + 1);
% Sample from the Pearson system with the provided moments
U_moments = {0, sqrt(U_m2), U_m3/U_m2^(3/2), kurtosis};
pearson_sample = pearsrnd(U_moments{:}, numberOfPearsonSamples, 1);
% Compute empirical CDF and get the 95% quantile
[mmd_empirical_cdf, mmd_empirical_cdf_x] = ecdf(pearson_sample);
test_threshold = ...
    mmd_empirical_cdf_x(find(mmd_empirical_cdf < 1 - alpha, 1, 'last'));

end