function [mmd, test_threshold] = ...
    mmd_spectrum_test(x1, x2, kernel, alpha, numberOfEigenvalues, numberOfNullSamples)
% mmd_spectrum_test computes the MMD value for samples x1 and x2, using 
% kernel as the kernel function and it then uses the eigenvalues/spectrum 
% approximation to compute a threshold for performing hypothesis testing 
% using parameter alpha as the significance level. The null hypothesis 
% being tested is assuming that x1 and x2 were generated from the same 
% distribution. Note that this test computes and uses the biased MMD 
% estimate. This method uses the eigenvalues of the joint kernel matrix 
% over both samples to obtain a consistent estimate of the null 
% distribution. It uses the numberOfEigenvalues largest eigenvalues and 
% samples numberOfNullSamples samples from the approximated null 
% distribution to compute the testing threshold.

n = size(x1, 1);

% Compute a biased MMD estimate
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
mmd = sum(sum(mmd_sum_terms)) / (n^2);

% Draw samples from null distribution
kernels_matrix = [K KL; KL' L];
H = eye(2*n) - ones(2*n, 2*n) / (2 * n);
kernels_matrix = H * kernels_matrix * H;
largest_eigenvalues = eigs(kernels_matrix, numberOfEigenvalues);
largest_eigenvalues = abs(largest_eigenvalues) / (2*n);
% Sample from the null spectral MMD distribution
null_spectral_MMD_samples = zeros(1, numberOfNullSamples);
for i = 1:numberOfNullSamples
  null_spectral_MMD_samples(i) = ...
      2 * sum(largest_eigenvalues ...
      .* (randn(length(largest_eigenvalues), 1)) .^2);
end
% Compute the test threshold
null_spectral_MMD_samples = sort(null_spectral_MMD_samples);
test_threshold = ...
    null_spectral_MMD_samples(round((1 - alpha) * numberOfNullSamples)) / n;

end